@@ -5,18 +5,19 @@ from logging import Logger, basicConfig, getLogger
 from typing import Any, Callable, Coroutine, List, Optional, Union, Dict
 
 from .api.cache import Cache, Item
-from .api.error import InteractionException, JSONException
-from .api.errors import *
+from .api.exceptions import *
 from .api.gateway import DiscordWebSocket, ReconnectWebSocket
 from .api.http import HTTPClient
-from .api.models.guild import Guild
-from .api.models.intents import Intents
-from .api.models.team import Application
-from .base import Data
 from .backoff import ExponentialBackoff
 from .enums import ApplicationCommandType
-from .models.command import ApplicationCommand, Option
-from .context import InteractionContext
+from .models import Intents
+from .models import ClientUser, Guild
+from .models import ApplicationCommand, Option
+from .exceptions import *
+
+from .context import Context, InteractionContext
+
+from . import __logger__
 
 __all__ = (
     'Client',
@@ -25,7 +26,7 @@ __all__ = (
 # TODO: Find a better way to call on the cache
 cache = Cache()
 
-basicConfig(level=Data.LOGGER)
+basicConfig(level=__logger__)
 log: Logger = getLogger("client")
 
 
@@ -80,8 +81,11 @@ class Client:
     """
     A class representing the client connection to Discord's gateway and API via. WebSocket and HTTP.
 
-    :ivar DiscordWebSocket ws: The Websocket we connect to
     :ivar asyncio.AbstractEventLoop loop: The main overall asynchronous coroutine loop in effect.
+    :ivar discord.models.Intents intents: Intents to apply when logging.
+    :ivar str proxy: Proxy for http library.
+    :ivar str proxy_auth: Auth to login into proxy.
+    :ivar bool unsync_clock: Assume unsynced clock.
     """
 
     def __init__(self, *, loop: Optional[asyncio.AbstractEventLoop] = None, **options: Any,) -> None:
@@ -97,6 +101,8 @@ class Client:
         unsync_clock = options.pop('assume_unsync_clock', True)
         self.http = HTTPClient(connector, proxy=proxy, proxy_auth=proxy_auth, unsync_clock=unsync_clock, loop=self.loop)
 
+        self.intents = options.pop('intents', Intents.all())
+
         self._handlers = {
             'ready': self._handle_ready
         }
@@ -146,7 +152,7 @@ class Client:
         """
 
         log.info('logging in using static token')
-        self.me = await self.http.static_login(token.strip(), bot=bot)
+        self.user = await self.http.static_login(token.strip(), bot=bot)
 
     async def connect(self, *, reconnect=True):
         """|coro|
@@ -259,6 +265,26 @@ class Client:
         self._ready.clear()
         self.http.recreate()
 
+    async def get_self_info(self):
+        """|coro|
+
+        After connecting to client and socket fill information of user.
+
+        """
+
+        # Fill user info
+        if(self.user):
+            self.user = ClientUser(client=self, data=self.user)
+        # Get guilds too
+        self.guilds = list()
+        temp_guilds = await self.http.get_self_guilds()
+        for guild in temp_guilds:
+            full_guild = await self.http.get_guild(guild['id'])
+            self.guilds.append(Guild(client=self, data=full_guild))
+            await asyncio.sleep(1) # wait just a second to make sure we dont destroy the request rate limit per second
+            # this waiting may be specially bad if a bot is in more than a few servers and if so I suggest
+            # moving this functionality to a separate task where to build the internal cache.
+
     async def start(self, *args, **kwargs):
         """|coro|
 
@@ -276,6 +302,9 @@ class Client:
             raise TypeError("unexpected keyword argument(s) %s" % list(kwargs.keys()))
 
         await self.login(*args, bot=bot)
+        await self.get_self_info()
+        self.dispatch('ready')
+        # .connect locks the execution, so anything you gotta do, before this line
         await self.connect(reconnect=reconnect)
 
     def run(self, *args, **kwargs):
@@ -355,8 +384,6 @@ class Client:
 
         The default implementation sleeps for 5 seconds.
 
-        .. versionadded:: 1.4
-
         Parameters
         ------------
         shard_id: :class:`int`
@@ -374,12 +401,12 @@ class Client:
     async def _syncer(self, guilds):
         await self.ws.request_sync(guilds)
 
-    def _handle_ready(self, _):
+    def _handle_ready(self, event, data: dict = None):
         self._ready.set()
         log.debug(f"Im Handling ready and setting up {self._application_commands}")
         # Dispatch all pending application_commands
         for name, data in self._application_commands.items():
-            self._setup_application_command(name, self.me['id'], data)
+            self._setup_application_command(name, self.user.id, data)
         asyncio.create_task(self.synchronize_commands())
 
     def dispatch_raw(self, event, *args, **kwargs):
@@ -397,7 +424,7 @@ class Client:
 
         app_context = None
         if data:
-            app_context = InteractionContext(**data)
+            app_context = Context(**data)
 
         if event in self._handlers:
             self._handlers[event](app_context)
@@ -496,7 +523,7 @@ class Client:
 
         for guild in guilds:
             await asyncio.sleep(2)
-            commands = await self.http.get_application_command(self.me['id'], guild['id'])
+            commands = await self.http.get_application_command(self.user.id, guild['id'])
             
             change: list = []
 
@@ -508,7 +535,7 @@ class Client:
                 else:
                     await asyncio.sleep(3)
                     await self.http.delete_application_command(
-                        application_id=self.me['id'],
+                        application_id=self.user.id,
                         command_id=command["id"],
                         guild_id=command.get("guild_id"),
                     )
@@ -517,7 +544,7 @@ class Client:
                 log.debug(f"Updated command {command['id']}.")
                 await asyncio.sleep(3)
                 self.http.edit_application_command(
-                    application_id=self.me['id'],
+                    application_id=self.user.id,
                     data=command["data"],
                     command_id=command["id"],
                     guild_id=command.get("guild_id"),
@@ -558,9 +585,10 @@ class Client:
         type: Optional[Union[str, int, ApplicationCommandType]] = ApplicationCommandType.CHAT_INPUT,
         name: Optional[str] = None,
         description: Optional[str] = None,
-        scope: Optional[Union[int, Guild, List[int], List[Guild]]] = None,
+        scope: Optional[Union[int, List[int]]] = None,
         options: Optional[List[Option]] = None,
-        default_permission: Optional[bool] = None
+        default_permission: Optional[bool] = None,
+        permissions: Optional[List[str]] = None
     ) -> Callable[..., Any]:
         """
         A decorator for registering an application command to the Discord API,
@@ -592,14 +620,22 @@ class Client:
             if not asyncio.iscoroutinefunction(coro):
                 raise TypeError('event registered must be a coroutine function')
 
-            # What we're going to do here is take the function and push it back until we're logged in.
-            # Once we're logged in we get all bot information and register the functions.
-
+            # Store the function in the bot
             setattr(self, f"{name}_{type}", coro)
-            self._application_commands[f"{name}_{type}"] = \
-            {'type': type, 'name': name, 'description': description, 'scope': scope, 
-                'options': options, 'default_permission': default_permission}
-            log.debug('%s has successfully been registered as an application command', coro.__name__)
+            application_data = {'type': type, 'name': name, 'description': description, 'scope': scope, 
+                    'options': options, 'default_permission': default_permission, 'permissions': permissions}
+
+            # We need to verify whether the bot is logged in or not.
+            if self.is_ready():
+                # Just setup the command
+                self._setup_application_command(name, self.user.id, application_data)
+
+            else:
+                # We delegate the command to after the bot is logged in
+                self._application_commands[f"{name}_{type}"] = application_data
+                
+            log.debug('%s has successfully been queued as an application command', coro.__name__)
+
             return coro
 
         return decorator
@@ -609,17 +645,19 @@ class Client:
         _description: str = "" if application_data.get("description") is None else application_data["description"]
         _options: list = [] if application_data.get("options") is None else application_data["options"]
         _default_permission: bool = True if application_data.get("default_permission") is None else application_data["default_permission"]
-        # _permissions: list = [] if permissions is None else permissions
+        _permissions: list = [] if application_data['permissions'] is None else application_data['permissions']
         _scope: list = []
 
         if isinstance(application_data.get("description"), list):
-            if all(isinstance(x, Guild) for x in scope):
-                _scope.append(guild.id for guild in scope)
-            elif all(isinstance(x, int) for x in scope):
+            if all(isinstance(x, int) for x in scope):
                 _scope.append(guild for guild in scope)
         else:
             _scope.append(application_data.get("scope"))
 
+        # Double check just so users don't fuck up
+        if len(_permissions) > 0:
+            _default_permission = False
+
         for guild in _scope:
             payload: ApplicationCommand = ApplicationCommand(
                 type=_type,
@@ -629,11 +667,21 @@ class Client:
                 default_permission=_default_permission,
             )
 
-            request = self.http.create_application_command(
-                application_id=application_id, data=payload._json, guild_id=guild
-            )
+            asyncio.create_task(self._register_application_command_and_permissions(application_id, guild, payload, _permissions))
 
-            asyncio.create_task(request)
+    async def _register_application_command_and_permissions(self, application_id, guild, payload, permissions):
+        request = await self.http.create_application_command(
+            application_id=application_id, data=payload._json, guild_id=guild
+        )
+
+        command_id = request['id']
+
+        if len(permissions) > 0:
+                data = {"permissions": permissions}
+
+                request = await self.http.edit_application_command_permissions(
+                    application_id=application_id, guild_id=guild, command_id=command_id, data=data
+                )
 
     async def raw_socket_create(self, data: dict) -> None:
         # TODO: doctype what this does
